'use strict';

Object.defineProperty(exports, '__esModule', {
    value: true
});

var _get = function get(_x4, _x5, _x6) { var _again = true; _function: while (_again) { var object = _x4, property = _x5, receiver = _x6; desc = parent = getter = undefined; _again = false; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { _x4 = parent; _x5 = property; _x6 = receiver; _again = true; continue _function; } } else if ('value' in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } } };

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ('value' in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _inherits(subClass, superClass) { if (typeof superClass !== 'function' && superClass !== null) { throw new TypeError('Super expression must either be null or a function, not ' + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) subClass.__proto__ = superClass; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

var _d2LibCheck = require('d2/lib/check');

var _d2LibUtils = require('d2/lib/utils');

var _d2ModelModel = require('d2/model/Model');

var _d2ModelModel2 = _interopRequireDefault(_d2ModelModel);

var _d2ModelModelCollection = require('d2/model/ModelCollection');

var _d2ModelModelCollection2 = _interopRequireDefault(_d2ModelModelCollection);

var _d2LibSchemaTypes = require('d2/lib/SchemaTypes');

var _d2LibSchemaTypes2 = _interopRequireDefault(_d2LibSchemaTypes);

var _d2ModelFilters = require('d2/model/Filters');

var _d2ModelFilters2 = _interopRequireDefault(_d2ModelFilters);

var _d2ModelModelBase = require('d2/model/ModelBase');

/**
 * @class ModelDefinition
 *
 * @description
 * Definition of a Model. Basically this object contains the meta data related to the Model. Like `name`, `apiEndPoint`, `modelValidation`, etc.
 * It also has methods to create and load Models that are based on this definition. The Data element `ModelDefinition` would be used to create Data Element `Model`s
 *
 * Note: ModelDefinition has a property `api` that is used for the communication with the dhis2 api. The value of this
 * property is an instance of `Api`.
 */

var ModelDefinition = (function () {
    function ModelDefinition(modelName, modelNamePlural, modelOptions, properties, validations) {
        _classCallCheck(this, ModelDefinition);

        (0, _d2LibCheck.checkType)(modelName, 'string');
        (0, _d2LibCheck.checkType)(modelNamePlural, 'string', 'Plural');

        (0, _d2LibUtils.addLockedProperty)(this, 'name', modelName);
        (0, _d2LibUtils.addLockedProperty)(this, 'plural', modelNamePlural);
        (0, _d2LibUtils.addLockedProperty)(this, 'isMetaData', modelOptions && modelOptions.metadata || false);
        (0, _d2LibUtils.addLockedProperty)(this, 'apiEndpoint', modelOptions && modelOptions.apiEndpoint);
        (0, _d2LibUtils.addLockedProperty)(this, 'modelProperties', properties);
        (0, _d2LibUtils.addLockedProperty)(this, 'modelValidations', validations);

        this.filters = _d2ModelFilters2['default'].getFilters(this);
    }

    _createClass(ModelDefinition, [{
        key: 'filter',
        value: function filter() {
            return this.clone().filters;
        }
    }, {
        key: 'create',

        /**
         * @method create
         *
         * @param {Object} [data] Datavalues that should be loaded into the model.
         *
         * @returns {Model} Returns the newly created model instance.
         *
         * @description
         * Creates a fresh Model instance based on the `ModelDefinition`. If data is passed into the method that
         * data will be loaded into the matching properties of the model.
         *
         * ```js
         * dataElement.create({name: 'ANC', id: 'd2sf33s3ssf'});
         * ```
         */
        value: function create(data) {
            var model = _d2ModelModel2['default'].create(this);

            if (data) {
                //Set the datavalues onto the model directly
                Object.keys(model).forEach(function (key) {
                    model.dataValues[key] = data[key];
                });
            }

            return model;
        }
    }, {
        key: 'clone',
        value: function clone() {
            var ModelDefinitionPrototype = Object.getPrototypeOf(this);
            var clonedDefinition = Object.create(ModelDefinitionPrototype);
            var priorFilters = this.filters.filters;

            clonedDefinition = (0, _d2LibUtils.copyOwnProperties)(clonedDefinition, this);
            clonedDefinition.filters = _d2ModelFilters2['default'].getFilters(clonedDefinition);
            clonedDefinition.filters.filters = priorFilters.map(function (filter) {
                return filter;
            });

            return clonedDefinition;
        }
    }, {
        key: 'get',

        /**
         * @method get
         *
         * @param {String} identifier
         * @param {Object} [queryParams={fields: ':all'}] Query parameters that should be passed to the GET query.
         * @returns {Promise} Resolves with a `Model` instance or an error message.
         *
         * @description
         * Get a `Model` instance from the api loaded with data that relates to `identifier`.
         * This will do an API call and return a Promise that resolves with a `Model` or rejects with the api error message.
         *
         * ```js
         * //Do a get request for the dataElement with given id (d2sf33s3ssf) and print it's name
         * //when that request is complete and the model is loaded.
         * dataElement.get('d2sf33s3ssf')
         *   .then(model => console.log(model.name));
         * ```
         */
        value: function get(identifier) {
            var _this = this;

            var queryParams = arguments[1] === undefined ? { fields: ':all' } : arguments[1];

            (0, _d2LibCheck.checkDefined)(identifier, 'Identifier');

            //TODO: should throw error if API has not been defined
            return this.api.get([this.apiEndpoint, identifier].join('/'), queryParams).then(function (data) {
                return _this.create(data);
            })['catch'](function (response) {
                return Promise.reject(response.data);
            });
        }
    }, {
        key: 'list',

        /**
         * @method list
         *
         * @param {Object} [queryParams={fields: ':all'}] Query parameters that should be passed to the GET query.
         * @returns {ModelCollection} Collection of model objects of the `ModelDefinition` type.
         *
         * @description
         * Loads a list of models.
         *
         * ```js
         * // Loads a list of models and prints their name.
         * dataElement.list()
         *   .then(modelCollection => {
         *     modelCollection.forEach(model => console.log(model.name));
         *   });
         * ```
         */
        value: function list() {
            var _this2 = this;

            var queryParams = arguments[0] === undefined ? { fields: ':all' } : arguments[0];

            var definedFilters = this.filters.getFilters();
            if (!(0, _d2LibCheck.isDefined)(queryParams.filter) && definedFilters.length) {
                queryParams.filter = definedFilters;
            }

            return this.api.get(this.apiEndpoint, queryParams).then(function (data) {
                return _d2ModelModelCollection2['default'].create(_this2, data[_this2.plural].map(function (data) {
                    return _this2.create(data);
                }), data.pager);
            });
        }
    }, {
        key: 'save',

        /**
         * @method save
         *
         * @param {Model} model The model that should be saved to the server.
         * @returns {Promise} A promise which resolves when the save was successful
         * or rejects when it failed. The promise will resolve with the data that is
         * returned from the server.
         *
         * @description
         * This method is used by the `Model` instances to save the model when calling `model.save()`.
         *
         * @note {warning} This should generally not be accessed directly.
         */
        //TODO: check the return status of the save to see if it was actually successful and not ignored
        value: function save(model) {
            var isAnUpdate = function isAnUpdate(model) {
                return !!model.id;
            };
            if (isAnUpdate(model)) {
                return this.api.update(model.dataValues.href, this.getOwnedPropertyJSON(model));
            } else {
                //Its a new object
                return this.api.post(this.apiEndpoint, this.getOwnedPropertyJSON(model));
            }
        }
    }, {
        key: 'getOwnedPropertyJSON',
        value: function getOwnedPropertyJSON(model) {
            var objectToSave = {};
            var ownedProperties = this.getOwnedPropertyNames();

            Object.keys(this.modelValidations).forEach(function (propertyName) {
                if (ownedProperties.includes(propertyName)) {
                    if (model.dataValues[propertyName]) {
                        objectToSave[propertyName] = model.dataValues[propertyName];
                    }
                }
            });

            return objectToSave;
        }
    }, {
        key: 'getOwnedPropertyNames',

        /**
         * @method getOwnedPropertyNames
         *
         * @returns {String[]} Returns an array of property names.
         *
         * @description
         * This method returns a list of property names that that are defined
         * as "owner" properties on this schema. This means these properties are used
         * when saving the model to the server.
         *
         * ```js
         * dataElement.getOwnedPropertyNames()
         * ```
         */
        value: function getOwnedPropertyNames() {
            var _this3 = this;

            return Object.keys(this.modelValidations).filter(function (propertyName) {
                return _this3.modelValidations[propertyName].owner;
            });
        }
    }], [{
        key: 'createFromSchema',

        /**
         * @method createFromSchema
         * @static
         *
         * @returns {ModelDefinition} Frozen model definition object.
         *
         * @description
         * This method creates a new `ModelDefinition` based on a JSON structure called
         * a schema. A schema represents the structure of a domain model as it is
         * required by DHIS. Since these schemas can not be altered on the server from
         * the modelDefinition is frozen to prevent accidental changes to the definition.
         *
         * ```js
         * ModelDefinition.createFromSchema(schemaDefinition);
         * ```
         *
         * @note {info} An example of a schema definition can be found on
         * https://apps.dhis2.org/demo/api/schemas/dataElement
         */
        value: function createFromSchema(schema) {
            var ModelDefinitionClass = undefined;
            (0, _d2LibCheck.checkType)(schema, Object, 'Schema');

            if (typeof ModelDefinition.specialClasses[schema.name] === 'function') {
                ModelDefinitionClass = ModelDefinition.specialClasses[schema.name];
            } else {
                ModelDefinitionClass = ModelDefinition;
            }

            return Object.freeze(new ModelDefinitionClass(schema.name, schema.plural, schema, Object.freeze(createPropertiesObject(schema.properties)), Object.freeze(createValidations(schema.properties))));
        }
    }]);

    return ModelDefinition;
})();

var UserModelDefinition = (function (_ModelDefinition) {
    function UserModelDefinition() {
        _classCallCheck(this, UserModelDefinition);

        if (_ModelDefinition != null) {
            _ModelDefinition.apply(this, arguments);
        }
    }

    _inherits(UserModelDefinition, _ModelDefinition);

    _createClass(UserModelDefinition, [{
        key: 'get',
        value: function get(identifier) {
            var queryParams = arguments[1] === undefined ? { fields: ':all,userCredentials[:owner]' } : arguments[1];

            return _get(Object.getPrototypeOf(UserModelDefinition.prototype), 'get', this).call(this, identifier, queryParams);
        }
    }]);

    return UserModelDefinition;
})(ModelDefinition);

ModelDefinition.specialClasses = {
    user: UserModelDefinition
};

function createPropertiesObject(schemaProperties) {
    var propertiesObject = {};
    var createModelPropertyDescriptorOn = (0, _d2LibUtils.curry)(createModelPropertyDescriptor, propertiesObject);

    (schemaProperties || []).forEach(createModelPropertyDescriptorOn);

    return propertiesObject;
}

function createModelPropertyDescriptor(propertiesObject, schemaProperty) {
    var propertyName = schemaProperty.collection ? schemaProperty.collectionName : schemaProperty.name;
    var propertyDetails = {
        //Actual property descriptor properties
        configurable: false,
        enumerable: true,
        get: function get() {
            return this.dataValues[propertyName];
        }
    };

    //Only add a setter for writable properties
    if (schemaProperty.writable) {
        propertyDetails.set = function (value) {

            //TODO: Objects and Arrays are concidered unequal when their data is the same and therefore trigger a dirty
            if (!(0, _d2LibCheck.isObject)(value) && value !== this.dataValues[propertyName] || (0, _d2LibCheck.isObject)(value)) {
                this.dirty = true;
                this[_d2ModelModelBase.DIRTY_PROPERTY_LIST].add(propertyName);
                this.dataValues[propertyName] = value;
            }
        };
    }

    if (propertyName) {
        propertiesObject[propertyName] = propertyDetails;
    }
}

function createValidations(schemaProperties) {
    var validationsObject = {};
    var createModelPropertyOn = (0, _d2LibUtils.curry)(createValidationSetting, validationsObject);

    (schemaProperties || []).forEach(createModelPropertyOn);

    return validationsObject;
}

function createValidationSetting(validationObject, schemaProperty) {
    var propertyName = schemaProperty.collection ? schemaProperty.collectionName : schemaProperty.name;
    var validationDetails = {
        persisted: schemaProperty.persisted,
        type: _d2LibSchemaTypes2['default'].typeLookup(schemaProperty.propertyType),
        required: schemaProperty.required,
        min: schemaProperty.min,
        max: schemaProperty.max,
        owner: schemaProperty.owner,
        unique: schemaProperty.unique,
        writable: schemaProperty.writable,
        constants: schemaProperty.constants
    };

    //Add a referenceType to be able to get a hold of the reference objects model.
    if (validationDetails.type === 'REFERENCE' || validationDetails.type === 'COLLECTION' && schemaProperty.itemPropertyType === 'REFERENCE') {
        validationDetails.referenceType = getReferenceTypeFrom(schemaProperty);
    }

    if (propertyName) {
        validationObject[propertyName] = validationDetails;
    }

    function getReferenceTypeFrom(schemaProperty) {
        if (schemaProperty.href) {
            return schemaProperty.href.split('/').pop();
        }
    }
}

exports['default'] = ModelDefinition;
module.exports = exports['default'];